package exercise;

import java.util.List;
import java.util.Set;
import junit.framework.TestCase;
import org.jgrapht.Graph;
import org.jgrapht.alg.connectivity.KosarajuStrongConnectivityInspector;
import org.jgrapht.generate.GnpRandomGraphGenerator;
import org.jgrapht.graph.DefaultDirectedGraph;
import org.jgrapht.graph.DefaultEdge;
import org.jgrapht.util.SupplierUtil;
import org.junit.Test;


public class SccTests extends TestCase {

  @Test
  public void testCorrectStronglyConnectedComponentsOnExampleGraph() {
    Graph<Integer, DefaultEdge> graph =
            new DefaultDirectedGraph<Integer, DefaultEdge>(DefaultEdge.class);
    for (int i = 0; i < 8; i++) {
      graph.addVertex(i);
    }
    graph.addEdge(0, 1);
    graph.addEdge(1, 3);
    graph.addEdge(3, 3);
    graph.addEdge(3, 0);
    graph.addEdge(4, 3);
    graph.addEdge(4, 5);
    graph.addEdge(5, 6);
    graph.addEdge(6, 4);
    graph.addEdge(7, 5);
    graph.addEdge(7, 6);

    KosarajuStrongConnectivityInspector inspector = new KosarajuStrongConnectivityInspector(graph);
    List<Set<Integer>> sccsWithLibrary = inspector.stronglyConnectedSets();

    List<Set<Integer>> sccs = Exercise9.getStronglyConnectedComponents(graph);

    assertTrue(sccs.size() == sccsWithLibrary.size());
    for (Set<Integer> scc: sccsWithLibrary) {
      boolean found = false;
      for (Set<Integer> otherScc: sccs) {
        if (scc.equals(otherScc)) {
          found = true;
          break;
        }
      }
      assertTrue(found);
    }
  }
  
  @Test
  public void testCorrectStronglyConnectedComponentsOnRandomGraph() {
    GnpRandomGraphGenerator<Integer,DefaultEdge> generator = new GnpRandomGraphGenerator(10, 0.22);
    Graph<Integer, DefaultEdge> graph = 
            new DefaultDirectedGraph<Integer,
            DefaultEdge>(SupplierUtil.createIntegerSupplier(),
            SupplierUtil.createDefaultEdgeSupplier(), false);
    generator.generateGraph(graph, null);
    // Exercise9.writeGraph(graph, "graphs/", "randomGraph.dot");

    KosarajuStrongConnectivityInspector inspector = new KosarajuStrongConnectivityInspector(graph);
    List<Set<Integer>> sccsWithLibrary = inspector.stronglyConnectedSets();
    System.out.println("strongly connected components computed with library:");
    System.out.println(sccsWithLibrary);

    List<Set<Integer>> sccs = Exercise9.getStronglyConnectedComponents(graph);
    System.out.println("strongly connected components computed with my implementation:");
    System.out.println(sccs);

    assertTrue(sccs.size() == sccsWithLibrary.size());
    for (Set<Integer> scc: sccsWithLibrary) {
      boolean found = false;
      for (Set<Integer> otherScc: sccs) {
        if (scc.equals(otherScc)) {
          found = true;
          break;
        }
      }
      assertTrue(found);
    }
  }

}
